Graphics3D 800,600,32,2

Global scale%=36

Dim LevelArray$(scale,scale)
Dim CSV$(500)

SeedRnd 42

movespeed#=0.05
turnspeed#=5
mapscale=4
turnangle#=-90

; read textures
tex=LoadTexture("textures/wall.jpg")
wall=CreateQuad()
EntityFX wall,2
ScaleEntity wall,0.5,0.5,0.5
EntityTexture wall,tex
UpdateNormals wall
HideEntity wall

stex=LoadTexture("textures/secret.jpg")
swall=CreateQuad()
EntityFX swall,2
ScaleEntity swall,0.5,0.5,0.5
EntityTexture swall,stex
UpdateNormals swall
HideEntity swall

fl=CreatePlane(16)
ft=LoadTexture("textures/floor.jpg",1)
EntityTexture Fl,ft
PositionEntity fl,0,0,0
ScaleTexture ft,0.5,0.5

cl=CreatePlane(16)
PositionEntity cl,0,1,0
RotateEntity cl,180,0,0
ct=LoadTexture("textures/floor.jpg",1)
EntityTexture cl,ct

; prepare minimap
image=CreateImage(scale*mapscale,scale*mapscale)
SetBuffer ImageBuffer(image)

; read level
level=ReadFile("level.csv")

; read level title
title$=ReadLine(level)
SplitCSVLine(title,";")
title=CSV(0)

; skip next line
ReadLine(level)

; read leveldata and create minimap
part$=""
For y=0 To scale-1
	
	part$=ReadLine(level)
	SplitCSVLine(part,";")
	
	For x=0 To scale-1
		
		; empty fields become spaces
		If CSV(x)="" Then CSV(x)=" "
		LevelArray(x,y)=CSV(x)
		
		; draw minimap
		Select Left(CSV(x),1)
				
			Case " ":	Color 128,128,96 : Rect x*mapscale,y*mapscale,mapscale,mapscale,1
			Case "S":	Color 192,96,192 : Rect x*mapscale,y*mapscale,mapscale,mapscale,1
			Case "W":	Color 32,32,32 : Rect x*mapscale,y*mapscale,mapscale,mapscale,1
				
				
		End Select
		
	Next
	
Next

SetBuffer BackBuffer()

; build level
For y=0 To scale-1
	
	For x=0 To scale-1
		
		If x>0 And x<scale And y>0 And y<scale Then
			
			PlaceWall(x,y,"W",wall)
			PlaceWall(x,y,"S",swall)
			
		EndIf
		
	Next

Next

; Camera
cam=CreateCamera()
PositionEntity cam,2,0.5,-7
CameraRange cam,0.001,16
CameraFogMode cam,True
CameraFogColor cam,0,0,0
CameraFogRange cam,0,10
CameraZoom cam,1.0/Tan(90.0/2.0)
CameraViewport cam,0,0,800,600
RotateEntity cam,0,90,0
RotateEntity cam,0,turnangle,0

; Light
torch=CreateCube(cam)
ScaleEntity torch,0.1,0.1,0.1
PositionEntity torch,0,0,0
light=CreateLight(2,torch)
LightColor light,255,224,192
LightRange light,1
AmbientLight 8,8,0

moved#=0

Global FramePeriod%=1000/60
Global FrameTime%=MilliSecs()-FramePeriod

; Main Loop
While Not KeyHit(1)
	
	Local FrameElapsed%,FrameTicks%,FrameTween#,t%
	
	Repeat FrameElapsed=MilliSecs()-FrameTime Until FrameElapsed
	FrameTicks=FrameElapsed/FramePeriod
	FrameTween=Float(FrameElapsed Mod FramePeriod)/Float(FramePeriod)
	
	For t=1 To FrameTicks
		
		If KeyHit(57) Then wf=1-wf : WireFrame wf
		
		move#=KeyDown(200)-KeyDown(208) : If move<>0 And turning=0 Then moving=move
		turn#=KeyDown(203)-KeyDown(205) : If turn<>0 And moving=0 Then turning=turn
		
		If moving Then
			
			If Abs(moved)<=1 Then
				MoveEntity cam,0,0,movespeed*moving
				moved=moved+(movespeed*moving)
			Else
				moving=0
				moved=0
				PositionEntity cam,Int(EntityX(cam)),0.5,Int(EntityZ(cam))
			EndIf
			
		EndIf
		
		If turning Then
			
			turnangle=(turnangle+(turnspeed*turning))
			RotateEntity cam,0,turnangle,0
			If (turnangle Mod 90) = 0 Then turning=0
			
		EndIf
		
		FrameTime=FrameTime+FramePeriod
		If t=FrameTicks Then CaptureWorld
		
		UpdateWorld
		
	Next
	
	; tweened render
	RenderWorld FrameTween
	
	x=Int(EntityX(cam))
	y=Int(-EntityZ(cam))
	
	;WaitTimer timer
	
	Color 255,255,255
	Text 0, 0,"Level...: "+title
	Text 0,15,"Position: "+x+","+y
	Text 0,30,"Tris....: "+TrisRendered()
	
	sx=GraphicsWidth()-ImageWidth(image)
	
	DrawImage image,sx,0
	Color 255,0,0
	Rect sx+(x*mapscale),(y*mapscale),mapscale,mapscale,1
	
	Flip 0
	
Wend

End

Function CreateQuad(r%=255,g%=255,b%=255,alpha#=1.0,fx%=0,centered%=False)
	
	Local mesh%,surface%,v1%,v2%,v3%,v4%,s#
	
	If centered Then s#=0.5 Else s#=1.0
	
	mesh=CreateMesh()
	surface=CreateSurface(mesh)
	
	v1=AddVertex (surface,-s, s,0,1,0)
	v2=AddVertex (surface, s, s,0,0,0)
	v3=AddVertex (surface,-s,-s,0,1,1)
	v4=AddVertex (surface, s,-s,0,0,1)
	
	VertexColor surface,v1,r,g,b,alpha
	VertexColor surface,v3,r,g,b,alpha
	VertexColor surface,v2,r,g,b,alpha
	VertexColor surface,v4,r,g,b,alpha
	
	AddTriangle(surface,0,1,2)
	AddTriangle(surface,3,2,1)
	
	EntityFX mesh,fx
	
	FlipMesh mesh
	
	Return mesh
	
End Function

Function SplitCSVLine%(l$,delim$)
	param = 0
	CSV$(param) = ""
	quote = False
	For loopy = 1 To Len(l$)
		bit$ = Mid$(l$,loopy,1)
		
		If bit$=Chr$(34) Then
			If quote = False Then
				quote = True 
			Else
				quote = False
			End If
		ElseIf bit$=delim$ And quote = False Then
				; end of param
			param = param + 1
			CSV$(param) = ""
		Else
			CSV$(param)=CSV$(param)+bit$
		End If
	Next
	Return param 
End Function

Function PlaceWall(x%,y%,what$,mesh%)
	
	Local temp%
	
	If LevelArray(x,y-1)=what Then
		temp=CopyEntity(mesh)
		PositionEntity temp,x,0.5,-y+0.5
		RotateEntity temp,0,180,0
	EndIf
	
	If LevelArray(x-1,y)=what Then
		temp=CopyEntity(mesh)
		PositionEntity temp,x-0.5,0.5,-y
		RotateEntity temp,0,-90,0
	EndIf
	
	If LevelArray(x+1,y)=what Then
		temp=CopyEntity(mesh)
		PositionEntity temp,x+0.5,0.5,-y
		RotateEntity temp,0,90,0
	EndIf
	
	If LevelArray(x,y+1)=what Then
		temp=CopyEntity(mesh)
		PositionEntity temp,x,0.5,-y-0.5
		RotateEntity temp,0,0,0
	EndIf
	
End Function
;~IDEal Editor Parameters:
;~C#Blitz3D